import static java.lang.Math.*;


public class vectorFunc{

    /*
    */
    
    public static final vector add(final vector a, final vector b)
    {
        return new vector( a.x+b.x, a.y+b.y);
    }

    public static final vector sub(final vector a, final vector b)
    {
        return new vector( a.x-b.x, a.y-b.y);
    }

    public static final vector mul(final vector a, final vector b)
    {
        return new vector( a.x*b.x, a.y*b.y);
    }
    
    public static final vector div(final vector a, final vector b)
    {
        return new vector( a.x/b.x, a.y/b.y);
    }
    
    public static final vector neg(final vector a)
    {
        return new vector( -a.x, -a.y);
    }
    
    public static final vector limit(final vector a, final double lim)
    {
        vector b;
        if (a.x > lim)
            b = new vector(lim, a.y);
        else if (a.x < -lim)
            b = new vector(-lim, a.y);
        else
            b = new vector(a);
        if (a.y > lim)
            b = new vector(b.x, lim);
        else if (a.y < -lim)
            b = new vector(b.x, -lim);
        return b;
    }
    
    /*
    */
    
    public static final vector invscale(final vector a, final double scalar) 
    {
        if (scalar != 0)
        {
            return new vector( a.x/scalar, a.y/scalar);
        }
        else
        {
            return new vector(a.x, a.y);
        }
    }
    
    public static final vector scale(final vector a, final double scalar)
    {
        return new vector( a.x*scalar, a.y*scalar);
    }
    
    public static final double dot(final vector a, final vector b)
    {
        return (a.x*b.x) + (a.y*b.y);
    }
    
    public static final double cross(final vector a, final vector b)
    {
        return (a.x*b.y) - (a.y*b.x);
    }
    
    /*
    */
    
    public static final double magSQ(final vector a)
    {
        return (pow(a.x,2.0) + pow(a.y,2.0));
    }
    
    public static final double mag(final vector a)
    {
        return sqrt(magSQ(a));
    }
    
    public static final vector normalize(final vector a)
    {
        double scalar = mag(a);
        return new vector( a.x/scalar, a.y/scalar);
    }

    
    public static final double dist(final vector a, final vector b)
    {
        return mag(sub(a,b));
    }
    
    public static double distSQ(vector a, vector b)
    {
        return magSQ(sub(a,b));
    }
    
    /*
    */
    
    public static final vector rotate(final vector a, final vector r)
    {
        return new vector(a.x * r.y - a.y * r.x, a.x * r.x + a.y * r.y);
    }
    
    public static final vector fromRadian(final double radian)
    {
        return new vector( sin(radian), cos(radian));
    }
    
    public static final double toRadian(final vector a)
    {
        return (double)atan2(a.x,a.y);
    }
    
    /*
    */
    
    public static final double closestU(final vector a, final vector p1, final vector p2)
    {
        vector delta = sub(p2,p1);
        return dot(sub(a,p1),delta)/magSQ(delta);
    }
    
    public static final vector closest(final vector a, final vector p1, final vector p2, final double U)
    {
        return add(p1, scale(sub(p2,p1),U));
    }
    
    public static final vector closest(final vector a, final vector p1, final vector p2)
    {
        vector delta = sub(p2,p1);
        double u = dot(sub(a,p1), delta)/ magSQ(delta);
        return add(p1, scale(delta,u));
    }
    
    /*
    */
    
    public static final double parallelZero(final vector p1, final vector p2, final vector p3, final vector p4)
    {
        return cross(sub(p2,p1), sub(p4,p3));
    }
    
    public static final double intersectUa(final vector p1, final vector p2, final vector p3, final vector p4, final double pZero)
    {
        vector delta = sub(p4,p3);
        vector deltaWTF = sub(p1,p3);
        return cross(delta,deltaWTF)/pZero;
    }
    
    public static final double intersectUb(final vector p1, final vector p2, final vector p3, final vector p4, final double pZero)
    {
        vector delta = sub(p2,p1);
        vector deltaWTF = sub(p1,p3);
        return cross(delta,deltaWTF)/pZero;
    }
    
    public static final vector intersect(final vector p1, final vector p2, final vector p3, final vector p4)
    {
        double pZero = parallelZero(p1,p2,p3,p4);
        double Ua = intersectUa(p1,p2,p3,p4,pZero);
        //double Ub = intersectUb(p1,p2,p3,p4,pZero);
        return add(p1,scale(sub(p2,p1),Ua));
    }
    
    public static final vector intersect(final vector p1, final vector p2, final double Ua)
    {
        return add(p1,scale(sub(p2,p1),Ua));
    }
    
};
